<?php

namespace NoccyLabs\GdCanvas;

use GdImage;
use NoccyLabs\GdImage\Image;
use NoccyLabs\GdImage\ImageInterface;



class Canvas implements ImageInterface
{

    private int $width;

    private int $height;

    private int $background;

    private array $layers = [];

    private ? GdImage $composited = null;

    /**
     */
    public function __construct(int $width, int $height, int $background = 0xFFFFFF)
    {
        $this->width = $width;
        $this->height = $height;
        $this->background = $background;
    }

	/**
	 * @return \GdImage
	 */
	public function getGdImage(): \GdImage
    {
	}

    public function growCanvas(?int $top=null, ?int $right=null, ?int $bottom=null, ?int $left=null)
    {
        $this->width += $left + $right;
        $this->height += $top + $bottom;

        foreach ($this->layers as $layer) {
            $layer->move($left, $top);
        }
    }

    public function createLayer(?int $width=null, ?int $height=null)
    {
        $width = $width ?? $this->width;
        $height = $height ?? $this->height;

        $layer = new Layer($width, $height);

        $this->layers[] = $layer;

        return $layer;
    }


    protected function compositeLayers(): GdImage
    {
        $image = imagecreatetruecolor($this->width, $this->height);
        imagefill($image, 0, 0, $this->background);

        foreach ($this->layers as $layer) {

            $layerImage = imagecreatetruecolor($layer->getWidth(), $layer->getHeight());
            imagecopymerge($layerImage, $image, 0, 0, $layer->getX(), $layer->getY(), $layer->getWidth(), $layer->getHeight(), 100);

            $layer->composite($layerImage);

            printf("restoring rect [%d,%d+%d,%d] into canvas of [%d,%d]\n", $layer->getX(), $layer->getY(), $layer->getWidth(), $layer->getWidth(), $this->width, $this->height);
            imagecopymerge($image, $layerImage, $layer->getX(), $layer->getY(), 0, 0, $layer->getWidth(), $layer->getHeight(), 100);
            imagedestroy($layerImage);
        }

        return $image;
    }

    public function saveAs(string $filename, bool $refresh=false)
    {
        if (!$this->composited || $refresh) {
            $this->composited = $this->compositeLayers();
        }

        imagejpeg($this->composited, $filename);
    }
}
