<?php

namespace NoccyLabs\GdCanvas;

use GdImage;
use NoccyLabs\GdCanvas\Drawable\Drawable;
use NoccyLabs\GdImage\Font\FontInterface;
use NoccyLabs\GdImage\Image;
use NoccyLabs\GdImage\ImageInterface;
use NoccyLabs\GdImage\Point;
use NoccyLabs\GdImage\Rect;

class Layer
{

    private int $width;

    private int $height;

    private int $x = 0;

    private int $y = 0;

    private array $objects = [];

    public function __construct(int $width, int $height)
    {
        $this->width = $width;
        $this->height = $height;

    }

    public function getX(): int
    {
        return $this->x;
    }

    public function getY(): int
    {
        return $this->y;
    }

    public function getWidth(): int
    {
        return $this->width;
    }

    public function getHeight(): int
    {
        return $this->height;
    }

    public function move(int $x, int $y, string $anchor="topleft", bool $absolute=false): self
    {
        if (!$absolute) {
            $x = $this->x + $x;
            $y = $this->y + $y;
        }

        switch ($anchor) {
            case 'topleft':
                $this->x = $x;
                $this->y = $y;
                break;
            case 'topright':
                $this->x = $x - $this->width;
                $this->y = $y;
                break;
            case 'bottomleft':
                $this->x = $x;
                $this->y = $y - $this->height;
                break;
            case 'bottomright':
                $this->x = $x - $this->width;
                $this->y = $y - $this->height;
                break;
            default:
                throw new \InvalidArgumentException("Invalid anchor");
        }

        return $this;
    }

    public function add(Drawable|ImageInterface $object, ?Rect $rect=null): DrawableObject
    {
        $obj = new DrawableObject($object, $rect);

        $this->objects[] = $obj;

        return $obj;
    }

    public function addText(FontInterface $font, ?Point $point=null, string $text, int $color): TextObject
    {
        $obj = new TextObject($font, $point, $text, $color);

        $this->objects[] = $obj;

        return $obj;
    }


    public function composite(GdImage $image)
    {
        foreach ($this->objects as $object) {
            $object->render($image);
        }
    }
}

