<?php

namespace NoccyLabs\GdCanvas;

use GdImage;
use NoccyLabs\GdCanvas\Drawable\Drawable;
use NoccyLabs\GdImage\Font\FontInterface;
use NoccyLabs\GdImage\Image;
use NoccyLabs\GdImage\ImageInterface;
use NoccyLabs\GdImage\Point;
use NoccyLabs\GdImage\Rect;

class TextObject extends CanvasObject
{
    private FontInterface $font;

    private Point $point;

    private string $text;

    private int $color;

    private string $anchor = 'topleft';


    public function __construct(FontInterface $font, Point $point, string $text, int $color)
    {
        $this->font = $font;
        $this->point = $point;
        $this->text = $text;
        $this->color = $color;
    }
    
    public function render(GdImage $image)
    {
        $options = [];

        [$x, $y] = [$this->point->x, $this->point->y];        
        [$tw, $th, $tx, $ty] = $this->font->measureString($this->text, $options);
        
        switch ($this->anchor) {
            case 'topleft':
                $ox = $x;
                $oy = $y;
                break;
            case 'topright':
                $ox = $x - $tw;
                $oy = $y;
                break;
            case 'bottomleft':
                $ox = $x;
                $oy = $y - $th;
                break;
            case 'bottomright':
                $ox = $x - $tw;
                $oy = $y - $th;
                break;
        }

        printf("renderstring [%d;%d] %s %06x\n", $ox, $oy, $this->text, $this->color);
        $this->font->renderString($image, $ox, $oy, $this->color, $this->text, $options);
        
    }
}

